searchData={"items":[{"type":"module","title":"Vaux","doc":"Provides functions to render Vaux components to html.\n\n    iex> defmodule HelloWorld do\n    ...>   import Vaux.Component\n    ...> \n    ...>   attr :title, :string\n    ...> \n    ...>   ~H\" {@title} \"vaux\n    ...> end\n    iex> Vaux.render(HelloWorld, %{\"title\" => \"Hello World\"})\n    {:ok, \" Hello World \"}","ref":"Vaux.html"},{"type":"type","title":"Vaux.attributes/0","doc":"","ref":"Vaux.html#t:attributes/0"},{"type":"function","title":"Vaux.render/3","doc":"Render a component to html\n\nAccepts a map where keys are strings as attributes and a map where keys are atoms as slots.\n\nReturns `{:ok, iodata()}` or `{:error, Vaux.RuntimeError.t()}` when an error \nis encountered during rendering.\n\nBy default a string is returned, but when \n\n    config :vaux, render_to_binary: false \n\nis set, an iolist is returned. Depending on the template complexity, this can \ngive a small performance boost. Note that this config setting is evaluated at \ncompile time.","ref":"Vaux.html#render/3"},{"type":"function","title":"Vaux.render!/3","doc":"Same as `render/3`, but raises an `Vaux.RuntimeError` exception when an error is encountered","ref":"Vaux.html#render!/3"},{"type":"type","title":"Vaux.slot_content/0","doc":"","ref":"Vaux.html#t:slot_content/0"},{"type":"behaviour","title":"Vaux.Component","doc":"Import this module to define a component\n\nIn addition to compiling the component template, the `sigil_H/2` macro \ncollects all defined attributes, variables and slots to define a struct that \nholds all this data as the component's state. This state struct is passed to \nthe compiled template and it's fields are accessible via the `@` assign syntax  \ninside the template.\n\nThe module's behaviour requires the callback functions `handle_state/1` and \n`render/1` to be implemented. However when defining a html template with \n`sigil_H/2`, both functions will be defined automatically. `handle_state/1` is \noverridable to make it possible to process the state struct before it gets \npassed to `render/1`.\n\n    iex> defmodule Component.StateExample do\n    ...>   import Vaux.Component\n    ...>  \n    ...>    @some_data_source %{name: \"Jan Jansen\", hobbies: ~w(cats drawing)}\n    ...>  \n    ...>    attr :title, :string\n    ...>    var :hobbies\n    ...>  \n    ...>    ~H\"\"\"\n    ...>       \n    ...>         {@title} \n    ...>         Current hobbies:{@hobbies} \n    ...>       \n    ...>    \"\"\"vaux\n    ...>\n    ...>    def handle_state(%__MODULE__{title: title} = state) do\n    ...>      %{name: name, hobbies: hobbies} = @some_data_source\n    ...>  \n    ...>      title = EEx.eval_string(title, assigns: [name: name])\n    ...>      hobbies = hobbies |> Enum.map(&String.capitalize/1) |> Enum.join(\", \")\n    ...>  \n    ...>      {:ok, %{state | title: title, hobbies: \" \" <> hobbies}}\n    ...>    end\n    ...>  end\n    iex> Vaux.render!(Component.StateExample, %{\"title\" => \"Hello <%= @name %>\"})\n    \"  Hello Jan Jansen  Current hobbies: Cats, Drawing  \"","ref":"Vaux.Component.html"},{"type":"macro","title":"Vaux.Component.attr/1","doc":"Define an attribute that can hold any value. See `attr/3` for more information about attributes.","ref":"Vaux.Component.html#attr/1"},{"type":"macro","title":"Vaux.Component.attr/3","doc":"Define an attribute\n\nAttributes, together with slots, provide the inputs to component templates. Attribute values are currently always html escaped. Vaux uses [JSV](https://hexdocs.pm/jsv/) for attribute validation. This means that most JSON Schema validation keywords are available for validating attributes.\n\nJSON Schema keywords are camelCased and can be used as such. However, the `attr/3` macro also supports Elixir friendly snake_case naming of JSON Schema keywords.\n\nThe following types are currently supported:\n\n  - `boolean`\n  - `object`\n  - `array`\n  - `number`\n  - `integer`\n  - `string`\n  - `true`\n  - `false`\n\nNote that the types `true` and `false` are different from type `boolean`. \n`true` means that any type will be accepted and `false` disallows any type. \nThese are mostly added for completeness, but especially `true` might be useful \nin some cases.\n\nOptions can be both applicator and validation keywords. Currently supported options are:\n\n  - `properties`\n  - `items`\n  - `contains`\n  - `maxLength`\n  - `minLength`\n  - `pattern`\n  - `exclusiveMaximum`\n  - `exclusiveMinimum`\n  - `maximum`\n  - `minimum`\n  - `multipleOf`\n  - `required`\n  - `maxItems`\n  - `minItems`\n  - `maxContains`\n  - `minContains`\n  - `uniqueItems`\n  - `description`\n  - `default`\n  - `format`\n\nA good resource to learn more about the use of these keywords is \n[www.learnjsonschema.com](https://www.learnjsonschema.com/2020-12/).\n\nThe `attr/3` macro provides some syntactic sugar for defining types. Instead of writing\n\n    attr :numbers, :array, items: :integer, required: true\n\nit is also possible to write\n\n    attr :numbers, {:array, :integer}, required: true\n\nWhen defining objects\n\n    attr :person, :object, properties: %{name: :string, age: :integer}\n\nit is also possible to write\n\n    attr :person, %{name: :string, age: :integer}\n\nAll validation options can be used when defining (sub)properties by using a tuple\n\n    attr :person, %{\n      name: {:string, min_length: 8, max_length: 16},\n      age: {:integer, minimum: 0}\n    }","ref":"Vaux.Component.html#attr/3"},{"type":"macro","title":"Vaux.Component.components/1","doc":"Convenience macro to declare components to use inside a template.\n\n    components My.Component\n\ngets translated to\n\n    require My.Component, as: Component\n\nA more complete example\n\n    defmodule MyComponent do\n      import Vaux.Component\n\n      components Some.{OtherComponent1, OtherComponent2}\n\n      components [\n        Some.Layout,\n        Another.Component\n      ]\n\n      ~H\"\"\"\n       \n         \n         \n         \n       \n      \"\"\"vaux\n    end","ref":"Vaux.Component.html#components/1"},{"type":"macro","title":"Vaux.Component.globals/1","doc":"Declare global attributes, including event handlers, that a component \noptionally accepts \n\nThe `globals/1` macro can be used if you want to allow setting global \nattributes on the template's first element. Use `true` to allow any global \nattribute and `false` to disallow all global attributes (default). When `:only` \nis passed as option, a list of global attribute names can be passed that are \nallowed. Finally, the `:except` option can be used to allow any global \nattribute except for the passed list of global attribute names.\n\nPassed global attributes overwrite already present attributes, except \nfor the `class` attribute. In that case passed classes will be added to \nexisting classes.\n\n    iex> defmodule GlobalsTest do\n    ...>   import Vaux.Component\n    ...> \n    ...>   attr :title, :string\n    ...>   globals only: ~w(id class onclick)\n    ...> \n    ...>   ~H\"\"\"\n    ...>    {@title} \n    ...>   \"\"\"vaux\n    ...> end\n    iex> Vaux.render(GlobalsTest, %{\"title\" => \"Hello World\", \"id\" => \"new-id\", \"class\" => \"myclass\", \"onclick\" => \"alert('Hi')\"})\n    {:ok, \" Hello World \"}\n\nSee https://developer.mozilla.org/en-US/docs/Web/HTML/Reference/Global_attributes for a complete list of global attributes that can be defined.","ref":"Vaux.Component.html#globals/1"},{"type":"callback","title":"Vaux.Component.handle_state/1","doc":"","ref":"Vaux.Component.html#c:handle_state/1"},{"type":"callback","title":"Vaux.Component.render/1","doc":"","ref":"Vaux.Component.html#c:render/1"},{"type":"macro","title":"Vaux.Component.sigil_H/2","doc":"Define a html template\n\nVaux templates support `{...}` for HTML-aware interpolation inside tag \nattributes and the body. `@field` can be used to access any field of the the \ntemplate's state struct. To access a root module constant, \n`@!constant` can be used. Note that Vaux templates require `vaux` as sigil \nmodifier in order to distinguish them from HEEx templates.\n\n    ~H\" {String.capitalize(@title)} \"vaux\n\nOnly variable interpolations are allowed inside `script` and `style` tags. \nThese are exressions in the form of `{@field}` or `{@!const}`. `{1 + @field}` is not valid \nvariable interpolation for example.\n\nAn extensive set of directives is available for expressing control flow, \niteration, template bindings and visibility within templates. Available control \nflow directives are:\n\n  - `:if`\n  - `:else`\n  - `:cond`\n  - `:case`\n  - `:clause`\n\nMost of these directives work like the equivalent in regular Elixir code. A \nnotable difference is that the `:cond` directive won't raise an exception when \nthere is no truthy condition, it simply skips rendering all elements with the \n`:cond` directive. However, when using the `:case` directive and there is no \nmatching `:clause`, an exception will be raised. This behaviour might change in \nfuture releases.\n\n    defmodule Component.DirectivesExample do\n      import Vaux.Component\n\n      attr :fruit, {:enum, ~w(apple banana pear orange)}\n      attr :count, :integer\n\n      ~H\"\"\"\n       \n        <!-- case expressions, just like in regular Elixir -->\n         \n           {String.upcase(@fruit)} \n           {String.reverse(@fruit)} \n\n          <!-- If the pattern is a string, you can ommit the curly braces  -->\n           {String.capitalize(@fruit)} \n           {String.replace(@fruit, \"g\", \"j\")} \n\n          <!-- Guards can be used too -->\n           Unexpected \n         \n\n        <!-- The first element with a truthy :cond expression gets rendered -->\n         = 5}>Too many \n         = 3}>Ok \n\n        <!-- :else can be used as the equivalent of `true -> ...` in a regular Elixir cond expression -->\n         Too little \n\n        <!-- :if (with or without a following :else) can be used too -->\n          \n       \n      \"\"\"vaux\n    end\n\n`:for` can be used for iterating. It supports a single Elixir `for` generator.\n\n     Number: {n} \n\nBy using the `:bind` and `:let` directives, it is possible to bind data in a \ntemplate and make it available to the consumer of the component. When using \nnamed slots, the `:let` directive can be used on the named template element.\n\n    iex> defmodule Component do\n    ...>   import Vaux.Component\n    ...>\n    ...>   attr :title, :string\n    ...>\n    ...>   ~H\"\"\"\n    ...>     \n    ...>   \"\"\"vaux  \n    ...> end\n    iex> defmodule Page do\n    ...>   import Vaux.Component\n    ...>\n    ...>   components Component\n    ...>\n    ...>   ~H\"\"\"\n    ...>    {upcased} \n    ...>   \"\"\"vaux  \n    ...> end\n    iex> Vaux.render!(Page)\n    \"HELLO WORLD\"\n\n\nFinally, the `:keep` directive can be used on template or slot elements to \nkeep them in the rendered output.","ref":"Vaux.Component.html#sigil_H/2"},{"type":"macro","title":"Vaux.Component.slot/1","doc":"Define a named slot\n\n    defmodule Layout do\n      import Vaux.Component\n\n      slot :content\n      slot :footer\n\n      ~H\"\"\"\n       \n         \n            \n         \n         \n            footer fallback content  \n         \n       \n      \"\"\"vaux\n    end\n\n    defmodule Page do\n      import Vaux.Component\n\n      components Layout\n\n      ~H\"\"\"\n       \n         \n           Hello World \n         \n         \n           \n             Hello World \n           \n                     \n       \n      \"\"\"vaux\n    end","ref":"Vaux.Component.html#slot/1"},{"type":"macro","title":"Vaux.Component.var/1","doc":"Define a variable\n\nA component variable can be either used as a constant, or in combination with \n`handle_state/1` as a place to store internal data that can be accessed inside \na template with the same `@` syntax as attributes.\n\n    iex> defmodule Hello do\n    ...>   import Vaux.Component\n    ...> \n    ...>   var title: \"Hello\"\n    ...> \n    ...>   ~H\" {@title} \"vaux\n    ...> end\n    iex> Vaux.render(Hello)\n    {:ok, \" Hello \"}","ref":"Vaux.Component.html#var/1"},{"type":"module","title":"Vaux.Root","doc":"Importing `Vaux.Root` lets a module define common elements for components to use.\n\nWhen at least one `const/1` or `components/1` definition is included in a root \nmodule, a `__using__/1` macro is created for the root module that allows it to be used \nin a component. \n\n    defmodule MyRoot do\n      import Vaux.Root\n\n      components [\n        Layout,\n        Component.{Example1, Example2}\n      ]\n\n      const title: \"Hello World\"\n    end\n\n    defmodule Page do\n      use MyRoot\n\n      ~H\"\"\"\n         \n           \n             \n           \n           \n             \n           \n         \n      \"\"\"vaux\n    end","ref":"Vaux.Root.html"},{"type":"macro","title":"Vaux.Root.components/1","doc":"See `Vaux.Component.components/1`","ref":"Vaux.Root.html#components/1"},{"type":"macro","title":"Vaux.Root.const/1","doc":"Define a constant\n\nThe defined constant is accessable in components via the `@!` template syntax.\n\nWhen the value is an argumentless function, it will be applied during compile time.\n\n    defmodule Constants do\n      import Vaux.Root\n\n      const answer: 42\n\n      const countries: fn ->\n        File.read!(\"priv/countries.json\") |> JSON.decode!()\n      end\n    end\n\n    defmodule Component do\n      use Constants\n\n      ~H\"\"\"\n        {name}  \n       Answer: {@!answer} \n      \"\"\"vaux\n    end","ref":"Vaux.Root.html#const/1"},{"type":"exception","title":"Vaux.CompileError","doc":"","ref":"Vaux.CompileError.html"},{"type":"function","title":"Vaux.CompileError.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Vaux.CompileError.html#message/1"},{"type":"exception","title":"Vaux.RuntimeError","doc":"","ref":"Vaux.RuntimeError.html"},{"type":"function","title":"Vaux.RuntimeError.errors/0","doc":"","ref":"Vaux.RuntimeError.html#errors/0"},{"type":"function","title":"Vaux.RuntimeError.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"Vaux.RuntimeError.html#message/1"},{"type":"type","title":"Vaux.RuntimeError.t/0","doc":"","ref":"Vaux.RuntimeError.html#t:t/0"},{"type":"extras","title":"Overview","doc":"# Overview","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `vaux` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:vaux, \"~> 0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Introduction - Overview","doc":"Vaux (rhymes with yo) provides composable html templates for Elixir. It uses \na customized verion of the excellent html parsing library \n[htmerl](https://hex.pm/packages/htmerl) to offer a simple, but still \nexpressive template syntax.\n\nIt builds upon HEEx template syntax, which means it offers good editor support \nout of the box.\n\nA minimal example looks like this:\n\n```elixir\n  defmodule Component.Example1 do\n    import Vaux.Component\n\n    attr :title, :string\n\n    ~H\"\"\"\n     {@title} \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Component.Example1, %{\"title\" => \"Hello World\"})\n  \" Hello World \"\n```\n\nIf you are familiar with Phoenix components, a Vaux component will look pretty \nsimilar, as it uses the same sigil and template expression syntax as HEEx \ntemplates. To make sure HEEx and Vaux templates can't be mixed up, Vaux \nrequires the `vaux` modifier for its `~H` sigil. A key difference is that Vaux, \nat least at the moment, only supports a single template per module.\n\nIn order to call another component, it needs to be known at compile time. Vaux \nprovides the `components/1` macro that both requires and aliases components:\n\n```elixir\n  defmodule Component.Example2 do\n    import Vaux.Component\n\n    attr :title, :string\n\n    ~H\"\"\"\n     {@title} \n    \"\"\"vaux\n  end\n\n  defmodule Page.Page1 do\n    import Vaux.Component\n\n    components Component.{Example1, Example2}\n\n    var title: \"Hello World\"\n\n    ~H\"\"\"\n     \n       \n         \n         \n         \n       \n       \n         \n       \n     \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Page.Page1)\n  \"     Hello World    Hello World   \"\n```","ref":"readme.html#introduction"},{"type":"extras","title":"Slots - Overview","doc":"Every component has a default slot that holds the element's content:\n\n```elixir\n\n  defmodule Layout.Layout1 do\n    import Vaux.Component\n\n    ~H\"\"\"\n     \n       \n          FALLBACK CONTENT  \n       \n     \n    \"\"\"vaux\n  end\n\n  defmodule Page.Page2 do\n    import Vaux.Component\n\n    components [\n      Component.Example1,\n      Layout.Layout1\n    ]\n\n    ~H\"\"\"\n     \n       \n     \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Page.Page2)\n  \"   Hello World   \"\n\n  defmodule Page.Page3 do\n    import Vaux.Component\n\n    components Layout.Layout1\n\n    ~H\"\"\"\n    <!-- Render fallback content if the component doesn't have any child elements -->\n      \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Page.Page3)\n  \"   FALLBACK CONTENT   \"\n```\n\nVaux also supports named slots. This allows you to easily separate page layout from page content.\n\nNamed slots need to be defined with the `slot/1` macro. This allows Vaux to \ncatch typos in the template at compile time and gives a component user a quick \noverview what slots are available.\n\n```elixir\n  defmodule Layout.Layout2 do\n    import Vaux.Component\n\n    slot :head\n    slot :body\n\n    ~H\"\"\"\n     \n       \n         \n         \n          \n       \n       \n          \n       \n     \n    \"\"\"vaux\n  end\n\n  defmodule Page.Page4 do\n    import Vaux.Component\n\n    components [\n      Component.{Example1, Example2},\n      Layout.Layout2\n    ]\n\n    var title: \"Hello World\"\n\n    ~H\"\"\"\n       \n         \n           \n         \n         \n           \n         \n       \n    \"\"\"vaux\n  end\n  \n  iex> Vaux.render!(Page.Page4)\n  \"     Hello World    Hello World   \"\n```","ref":"readme.html#slots"},{"type":"extras","title":"Directives - Overview","doc":"Vaux doesn't support block expressions, but it has an extensive set of \ndirectives to use:\n\n```elixir\n  defmodule Component.DirectivesExample do\n    import Vaux.Component\n\n    attr :fruit, {:enum, ~w(apple banana pear orange)}\n    attr :count, :integer\n\n    ~H\"\"\"\n     \n      <!-- case expressions, just like in regular Elixir -->\n       \n         {String.upcase(@fruit)} \n         {String.reverse(@fruit)} \n\n        <!-- If the pattern is a string, you can ommit the curly braces  -->\n         {String.capitalize(@fruit)} \n         {String.replace(@fruit, \"g\", \"j\")} \n\n        <!-- Guards can be used too -->\n         Unexpected \n       \n\n      <!-- Loops can be expressed with the :for directive -->\n       {number} \n\n      <!-- The first element with a truthy :cond expression gets rendered -->\n       = 5}>Too many \n       = 3}>Ok \n\n      <!-- :else can be used as the equivalent of `true -> ...` in a regular Elixir cond expression -->\n       Too little \n\n      <!-- :if can be used too -->\n        \n     \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Component.DirectivesExample, %{\"fruit\" => \"orange\", \"count\" => 3})\n  \"   oranje   1  2  3  Ok  \"\n```\n\n\n#### Applying directives to multiple elements\n\nIf you want to apply a directive to a list of elements, you can use the \n`template` element as a wrapper, as it won't get rendered by default (you can \nuse the `:keep` directive to keep te `template` element in the rendered output).\n\n```elixir\n  defmodule Component.Example3 do\n    import Vaux.Component\n\n    attr :fruit, {:enum, ~w(apple banana pear orange)}\n\n    ~H\"\"\"\n     \n        \n        \n     \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render!(Component.Example3, %{\"fruit\" => \"apple\"})\n  \"    \"\n```\n\n\n#### Using `:bind` and `:let` directives\n\nVaux templates also offer `:bind` and `:let` directives. These directives make \nit possible to bind data in a template and make it available to the consumer of \nthe component.\n\n```elixir\n     defmodule Component.BindingExample do\n       import Vaux.Component\n    \n       attr :title, :string\n    \n       ~H\"\"\"\n         \n       \"\"\"vaux  \n     end\n\n     defmodule Page.Page5 do\n       import Vaux.Component\n    \n       components Component.BindingExample\n    \n       ~H\"\"\"\n        {upcased} \n       \"\"\"vaux  \n     end\n\n    iex> Vaux.render!(Page)\n    \"HELLO WORLD\"\n```\n\nWhen using named slots, the `:let` directive can be used on the named template element.","ref":"readme.html#directives"},{"type":"extras","title":"Attribute validation - Overview","doc":"Vaux uses [JSV](https://hexdocs.pm/jsv/), a modern JSON Schema \nvalidation library. When defining an attribute with the `attr/3` macro, most \nJSON schema validation options can be used:\n\n```elixir\n  defmodule Component.Validations do\n    import Vaux.Component\n\n    # Both Elixir friendly snake_case and JSON Schema's camelCase notation can be used \n    attr :title, :string, min_length: 8, maxLength: 16, required: true\n    attr :count, :integer, required: true\n\n    # If the type of an array doesn't need extra validation, a shorthand notation can be used \n    attr :numbers1, :array, items: :integer\n    attr :numbers2, {:array, :integer}\n\n    # Shorthand notation for objects is available too\n    attr :person1, :object, properties: %{name: {:string, pattern: \"\\w+\\s+\\w+\"}, age: :integer}\n    attr :person2, %{name: {:string, pattern: \"\\w+\\s+\\w+\"}, age: :integer}\n\n    ~H\"\"vaux\n  end\n```","ref":"readme.html#attribute-validation"},{"type":"extras","title":"Global Attributes - Overview","doc":"Vaux supports passing global attributes to the first element of a component. \nThis can be used for example when you want to be able to set a class or event \nhandler on the component's root element.\n\n```elixir\n  defmodule GlobalsTest do\n    import Vaux.Component\n\n    attr :title, :string\n    globals only: ~w(id class onclick)\n\n    ~H\"\"\"\n     {@title} \n    \"\"\"vaux\n  end\n\n  iex> Vaux.render(GlobalsTest, %{\"title\" => \"Hello World\", \"id\" => \"new-id\", \"class\" => \"myclass\", \"onclick\" => \"alert('Hi')\"})\n  {:ok, \" Hello World \"}\n```\n\nSee [`globals/1`](Vaux.Component.html#globals/1) for more info.","ref":"readme.html#global-attributes"},{"type":"extras","title":"Vaux.Component behaviour and handle_state/1 callback - Overview","doc":"Every component implements the `Vaux.Component` behaviour. This behaviour \nrequires two functions to be implemented: `handle_state/1` and `render/1`. Both \nreceive a struct that is defined by the `sigil_H/2` macro. This struct contains \nall defined attributes, variables and slots. The `handle_state/1` function \nallows you to preprocess atributes, setup internal variables, etc. Finally, the \nreturned struct from `handle_state/1` is passed to the `render/1` function.\n\nThe `sigil_H/2` macro defines `render/1` and also provides an overridable \ndefault implementation for `handle_state/1`.\n\nThe main idea behind the `handle_state/1` callback is that it allows you to \nkeep most complex control flow and data transformations out of the template. \nFor top level components however, it can be convenient to treat the callback as \na type of view controller and let it fetch data from a data source itself. When \nto apply this strategy boils down to the same arguments when thinking about \nside effects in regular code: pure functions tend to be easier to compose and \nreason about, so that is a good default. However, making some key components \nresponsible for fetching data makes it simpler to reuse these components in \ndifferent contexts.\n\n```elixir\n  defmodule Component.StateExample do\n    import Vaux.Component\n\n    @some_data_source %{name: \"Jan Jansen\", hobbies: ~w(cats drawing)}\n\n    attr :title, :string\n    var :hobbies\n\n    ~H\"\"\"\n       \n         {@title} \n         Current hobbies:{@hobbies} \n       \n    \"\"\"vaux\n\n    def handle_state(%__MODULE__{title: title} = state) do\n      %{name: name, hobbies: hobbies} = @some_data_source\n\n      title = EEx.eval_string(title, assigns: [name: name])\n      hobbies = hobbies |> Enum.map(&String.capitalize/1) |> Enum.join(\", \")\n\n      {:ok, %{state | title: title, hobbies: \" \" <> hobbies}}\n    end\n  end\n\n  iex> Vaux.render!(Component.StateExample, %{\"title\" => \"Hello <%= @name %>\"})\n  \"  Hello Jan Jansen  Current hobbies: Cats, Drawing  \"\n```","ref":"readme.html#vaux-component-behaviour-and-handle_state-1-callback"},{"type":"extras","title":"Root modules - Overview","doc":"Vaux allows you to define root modules. These modules can be used to bundle \ncommon elements that components are able to use. \n\n```elixir\n  defmodule MyRoot do\n    import Vaux.Root\n\n    components [\n      Component.{Example1, Example2},\n      Layout.Layout2\n    ]\n\n    const title: \"Hello World\"\n  end\n\n  defmodule Page.Page6 do\n    use MyRoot\n\n    ~H\"\"\"\n       \n         \n           \n         \n         \n           \n         \n       \n    \"\"\"vaux\n  end\n  \n  iex> Vaux.render!(Page.Page6)\n  \"     Hello World    Hello World   \"\n```\n\nThe `const/1` macro allow you to define static data that is reused by multiple \ncomponents. These can be accessed in templates by the special `@!my_const` \nsyntax. The `components/1` macro works the same as in component definitions.\n\nWhen at least one `const/1` or `components/1` definition is included in a root \nmodule, a `__using__/1` macro is created for the root module that allows it to be used \nin a component.","ref":"readme.html#root-modules"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}